#!/bin/bash
# Limit extension usage to given time for each day

# Global variables
USAGE="daily_use.sh extension start|end|check [limit]"
SPOOLDIR=/var/spool/asterisk/phonepatch
LOCKEXEC="lockfile -1 -r2 -l5 -s2"
SPOOLNAME="limit-"
DATE=$(date +%Y%m%d)
SEC=$(date +%s)
START_TAG="start"
END_TAG="end"

##############
log() { echo $@ 1>&2; }

##############
error() { 
	log $@
	remove_lockfile $LOCKFILE
	exit 1
}

##############
get_usage()
{
	# $1: file
	CURRENT=0
	STATE="start"
	TEMP=$(tempfile)
	cat $1 | grep ^$DATE > $TEMP
	while read LINE; do
		OPT=$(echo $LINE | awk '{print $3}')
		if [ "$OPT" = $START_TAG ]; then
			SEC1=$(echo $LINE | awk '{print $2}')
			STATE="end"
		elif [ "$STATE" = "end" -a "$OPT" = $END_TAG ]; then
			SEC2=$(echo $LINE | awk '{print $2}')
			((CURRENT=CURRENT+SEC2-SEC1))
			STATE="start"
		fi
	done < $TEMP
	rm -f $TEMP
	log "get_usage: $CURRENT"
	RET=$CURRENT
}

####################
check_usage()
{
	FILE=$1
	LIMIT=$2
	get_usage $FILE
	if [ $RET -gt $LIMIT ]; then 
		echo "$DATE $SEC denied ($RET > $LIMIT)" >>$FILE
		error "daily usage reached: $RET > $LIMIT"
	fi
}

#############
create_lockfile() { $LOCKEXEC $1; }
remove_lockfile() { [ ! -z "$1" ] && rm -f $1; }

################
# Main
################

# Get parameters
EXTENSION=$1
OP=$2
LIMIT=$3

# Sanity checks
if [ -z "$EXTENSION" -o -z "$OP" ]; then error "usage: $USAGE"; fi
if [ "$OP" != "end" -a -z "$LIMIT" ]; then error "usage: $USAGE"; fi
if [ ! -d $SPOOLDIR ]; then error "spool directory not found: $SPOOLDIR"; fi
FILE="$SPOOLDIR/$SPOOLNAME$EXTENSION"
LOCKFILE="$FILE.lock"

# Lock the spool file
create_lockfile $LOCKFILE

# Log info
log "spool file: $FILE"
log "lock file: $LOCKFILE"

# Create spool file if not found
if [ ! -e $FILE ]; then touch $FILE; fi

# Available operations = start | end | check
if [ "$OP" = "start" ]; then 
	check_usage $FILE $LIMIT
	echo "$DATE $SEC $START_TAG" >> $FILE
elif [ "$OP" = "end" ]; then 
	echo "$DATE $SEC $END_TAG" >> $FILE
elif [ "$OP" = "check" ]; then 
	check_usage $FILE $LIMIT
else
	error "option not supported: $OP"
fi

remove_lockfile $LOCKFILE
log "done"
exit 0